#ifndef DEBTAGS_SERIALIZER_H
#define DEBTAGS_SERIALIZER_H

/** \file
 * Serializer for the Debtags tag collection
 */

/*
 * Copyright (C) 2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tagcoll/Exception.h>
#include <tagcoll/Serializer.h>

#include <debtags/Vocabulary.h>
#include <debtags/Tag.h>

namespace Debtags
{
class Tag;
class Vocabulary;
};

namespace Tagcoll
{

/**
 * Convert Facets to strings
 */
template<>
class Converter<Debtags::Facet, std::string>
{
public:
	std::string operator()(const Debtags::Facet& item) { return item.name(); }
	OpSet<std::string> operator()(const OpSet<Debtags::Facet>& items)
	{
		OpSet<std::string> res;
		for (OpSet<Debtags::Facet>::const_iterator i = items.begin();
				i != items.end(); i++)
			if (*i)
				res += (*this)(*i);
		return res;
	}
};

/**
 * Convert Tags to strings
 */
template<>
class Converter<Debtags::Tag, std::string>
{
public:
	std::string operator()(const Debtags::Tag& item) { return item.fullname(); }
	OpSet<std::string> operator()(const OpSet<Debtags::Tag>& items)
	{
		OpSet<std::string> res;
		for (OpSet<Debtags::Tag>::const_iterator i = items.begin();
				i != items.end(); i++)
			if (*i)
				res += (*this)(*i);
		return res;
	}
};

/**
 * Convert strings to Facets
 */
template<>
class Converter<std::string, Debtags::Facet>
{
protected:
	const Debtags::Vocabulary& voc;

public:
	Converter(const Debtags::Vocabulary& voc) : voc(voc) {}

	Debtags::Facet operator()(const std::string& item)
	{
		Debtags::Facet t = voc.getFacet(item);
		// FIXME: find a better default, or even return the invalid facet and
		// allow the caller to take care of it
		if (!t)
			return voc.getFacet("special");
		return t;
	}
	OpSet<Debtags::Facet> operator()(const OpSet<std::string>& items)
	{
		OpSet<Debtags::Facet> res;
		for (OpSet<std::string>::const_iterator i = items.begin();
				i != items.end(); i++)
		{
			Debtags::Facet t((*this)(*i));
			if (t)
				res += t;
		}
		return res;
	}
};

/**
 * Convert strings to Tags
 */
template<>
class Converter<std::string, Debtags::Tag>
{
protected:
	const Debtags::Vocabulary& voc;

public:
	Converter(const Debtags::Vocabulary& voc) : voc(voc) {}

	Debtags::Tag operator()(const std::string& item)
	{
		Debtags::Tag t = voc.getTag(item);
		if (!t)
			return voc.getTag("special::invalid-tag");
		return t;
	}
	OpSet<Debtags::Tag> operator()(const OpSet<std::string>& items)
	{
		OpSet<Debtags::Tag> res;
		for (OpSet<std::string>::const_iterator i = items.begin();
				i != items.end(); i++)
		{
			Debtags::Tag t((*this)(*i));
			if (t)
				res += t;
		}
		return res;
	}
};

};

// vim:set ts=4 sw=4:
#endif
