#ifndef DEBTAGS_TAG_H
#define DEBTAGS_TAG_H

/** \file
 * Debtags facets and tags
 */

/*
 * Copyright (C) 2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string>
#include <debtags/fdecls.h>

namespace Debtags
{

/**
 * Representation of a facet.
 *
 * Tagcoll::Facet represents a Facet with all its informations.  It is
 * implemented via a reference-counted smart pointer, so it can be passed
 * around freely and efficiently without worrying about memory management
 * issues.
 *
 * The class is normally instantiated using a Vocabulary:
 * \code
 * 	Facet facet = vocabulary.getFacet("made-of");
 * \endcode
 *
 * Facets can contain a "false" value, in which case using any of their
 * methonds produce a null-pointer dereference segfault.  The "null" facets are
 * useful as "none" return values:
 *
 * \code
 *    Facet facet = vocabulary.getFacet("made-of");
 *    if (!facet)
 *       throw ConsistencyCheckException("facet \"made-of\" has not been defined");
 * \endcode
 */
class Facet
{
protected:
	FacetImpl* impl;

	Facet(FacetImpl* impl) throw ();

public:
	Facet() throw ();
	Facet(const Facet& f) throw ();
	~Facet() throw ();
	Facet& operator=(const Facet& f) throw ();

	bool operator==(const Facet& f) const throw ();
	bool operator<(const Facet& f) const throw ();

	/**
	 * Return true if the facet is valid
	 */
	operator bool() const throw ();

	/**
	 * Return the name of the facet
	 */
	std::string name() const throw ();

	/**
	 * Return the short description of the facet
	 */
	std::string sdesc() const throw ();

	/**
	 * Return the long description of the facet
	 */
	std::string ldesc() const throw ();

	/**
	 * Return the full data record about the facet
	 */
	std::string fulldata() const throw ();

	/**
	 * Return the content of an arbitrary field
	 */
	std::string field(const std::string& name) const throw();

	/**
	 * Return true if the facet has a tag with the given name (name, not fullname)
	 */
	bool hasTag(const std::string& name) const throw ();

	/**
	 * Return the list of tags in this facet
	 */
	Tagcoll::OpSet<Tag> tags() const throw ();

#if 0
	Tag obtainTag(const std::string& name) throw ();
	Tag getTag(const std::string& name) const throw ();
#endif

	friend class Tag;
	friend class Vocabulary;
};

/**
 * Representation of a tag.
 *
 * Tagcoll::Tag represents a Tag with all its informations.  It is implemented
 * via a reference-counted smart pointer, so it can be passed around freely and
 * efficiently without worrying about memory management issues.
 *
 * The class is normally instantiated using a Vocabulary:
 * \code
 * 	Tag tag = vocabulary.getTag("made-of::lang:c++");
 * \endcode
 *
 * Tags can contain a "false" value, in which case using any of their
 * methonds produce a null-pointer dereference segfault.  The "null" tags are
 * useful as "none" return values:
 *
 * \code
 *    Tag tag = vocabulary.getTag("made-of");
 *    if (!tag)
 *       throw ConsistencyCheckException("tag \"mytag\" has not been defined");
 * \endcode
 */
class Tag
{
protected:
	TagImpl* impl;

	Tag(TagImpl* tag) throw ();

public:
	Tag() throw ();
	Tag(const Tag& f) throw ();
	~Tag() throw ();
	Tag& operator=(const Tag& f) throw ();

	bool operator==(const Tag& f) const throw ();
	bool operator<(const Tag& f) const throw ();

	operator bool() const throw ();

	Facet facet() const throw ();

	/**
	 * Return the name of the tag, without the facet:: prefix
	 */
	std::string name() const throw ();

	/**
	 * Return the name of the tag, with the facet:: prefix
	 */
	std::string fullname() const throw ();

	/**
	 * Return the short description of the tag
	 */
	std::string sdesc() const throw ();

	/**
	 * Return the long description of the tag
	 */
	std::string ldesc() const throw ();

	/**
	 * Return the full data record about the tag
	 */
	std::string fulldata() const throw ();

	/**
	 * Return the content of an arbitrary field
	 */
	std::string field(const std::string& name) const throw();

	friend class Vocabulary;
};

};

// vim:set ts=3 sw=3:
#endif
