#ifndef DEBTAGS_TAGDB_H
#define DEBTAGS_TAGDB_H

/*
 * System tag database
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tagcoll/Exception.h>
#include <tagcoll/ParserBase.h>
#include <tagcoll/TDBReadonlyDiskIndex.h>
#include <debtags/Tag.h>
#include <debtags/Serializer.h>

namespace Debtags
{
using namespace Tagcoll;

class Package;

/**
 * Access the local Debtags tag database.
 */
template<class ITEM>
class Tags : public TDBReadonlyDiskIndex<ITEM, Tag>
{
protected:
	Vocabulary voc;
	Converter<ITEM, std::string>& fromitem;
	Converter<Tag, std::string> fromtag;
	Converter<std::string, ITEM>& toitem;
	Converter<std::string, Tag> totag;

public:
	/** Create a new accessor for the Debtags database
	 *
	 * \param fromitem, fromtag, toitem, totag
	 * The serializers to use to serialize Package and Tag object into strings.
	 * 
	 * \param editable
	 * Specifies if recording of modifications should be enabled.  If editable
	 * is true, then the local state directory will be created when the object
	 * is instantiated.
	 */
	Tags(
			Converter<ITEM, std::string>& fromitem,
			Converter<std::string, ITEM>& toitem,
			bool editable = false);
	~Tags() {}

	/** @{
	 * Access the vocabulary
	 */
	const Vocabulary& vocabulary() const { return voc; }
	Vocabulary& vocabulary() { return voc; }
	/** @} */
	

	/**
	 * Check if the tag database has been created (i.e. if something
	 * equivalend to debtags update has been run)
	 */
	static bool hasTagDatabase();


	/**
	 * Save in the state storage directory a patch that can be used to turn
	 * the system database into the collection given
	 */
	void savePatch();

	/**
	 * Save in the state storage directory a patch to turn the system database
	 * into the collection given
	 */
	void savePatch(const PatchList<ITEM, Tag>& patch);

	/**
	 * Send to the central archive a patch that can be used to turn
	 * the system database into the collection given
	 */
	void sendPatch();

	/**
	 * Send the given patch to the central archive
	 */
	void sendPatch(const PatchList<ITEM, Tag>& patch);
	

	/** Output the current Debian tags database to a TagcollConsumer
	 * \note The collection is sent to 'cons' without merging repeated items
	 */
	void outputSystem(Tagcoll::Consumer<std::string, std::string>& cons);

	/** Output the current Debian tags database to a TagcollConsumer
	 * \note The collection is sent to 'cons' without merging repeated items
	 */
	void outputSystem(Tagcoll::Consumer<ITEM, Tag>& cons);

	/**
	 * Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	 * \note The collection is sent to 'cons' without merging repeated items
	 */
	void outputPatched(Tagcoll::Consumer<std::string, std::string>& cons);

	/**
	 * Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	 * \note The collection is sent to 'cons' without merging repeated items
	 */
	void outputPatched(Tagcoll::Consumer<ITEM, Tag>& cons);
};

};

// vim:set ts=4 sw=4:
#endif
