#ifndef DEBTAGS_UTILS_H
#define DEBTAGS_UTILS_H

/*
 * Ready-made utility classes for Debtags
 *
 * Copyright (C) 2003,2004,2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <debtags/Tags.h>
#include <debtags/Filters.h>
#include <debtags/Status.h>
#include <tagcoll/CardinalityStore.h>
#include <map>


namespace Debtags
{

template <class ITEM>
class SearchHelper
{
protected:
	static const int dist = 7;

	class CardCompute : public std::map<Tag, int>, public Tagcoll::Consumer<ITEM, Tag>
	{
		void consumeItemUntagged(const ITEM& item) {}
		void consumeItem(const ITEM& item, const Tagcoll::OpSet<Tag>& tags)
		{
			for (Tagcoll::OpSet<Tag>::const_iterator i = tags.begin();
					i != tags.end(); i++)
				(*this)[*i]++;
		}
	public:
		virtual ~CardCompute() {}
	};

	Tags<ITEM>& debtags;
	OpSet<ITEM> startPackages;
	OpSet<Tag> wanted;
	OpSet<Tag> unwanted;
	OpSet<ITEM> found;
	OpSet<Tag> pivotTags;
	CardCompute cardComputer;
	int cards[dist];
	Tag tags[dist];

	void storeTag(const Tag& tag, int card = -1, int idx = 0);

public:
	SearchHelper(Tags<ITEM>& debtags) : debtags(debtags) {}

	void clearStartPackages() { startPackages.clear(); }
	void addStartPackage(const ITEM& item) { startPackages += item; }

	void clearWanted() { wanted.clear(); }
	void addWanted(const Tag& tag) { wanted += tag; }
	const OpSet<Tag>& getWanted() const { return wanted; }

	void clearUnwanted() { unwanted.clear(); }
	void addUnwanted(const Tag& tag) { unwanted += tag; }
	const OpSet<Tag>& getUnwanted() const { return unwanted; }

	const OpSet<Tag>& getPivot() const { return pivotTags; }

	const OpSet<ITEM>& getFound() const { return found; }

	void compute();
};


/**
 * Compute "Special-case packages"
 *
 * Compute a list of those packages whose tag set is 'strange' compared to the
 * other packages.
 *
 * The list corresponds to those packages reported by 
 * debtags facetcoll | tagcoll findspecials
 */
template <class ITEM>
class Specials : public Tagcoll::Consumer<ITEM, Tag>
{
protected:
	Tagcoll::CardinalityStore<ITEM, Facet> coll;
	TagToFacet<ITEM> tagStripper;
	unsigned int maxPerGroup;
	bool computed;

	virtual void consumeItemUntagged(const ITEM& item)
	{
		tagStripper.consume(item);
	}
	virtual void consumeItem(const ITEM& item, const Tagcoll::OpSet<Tag>& tags)
	{
		tagStripper.consume(item, tags);
	}
	virtual void consumeItemsUntagged(const Tagcoll::OpSet<ITEM>& items)
	{
		tagStripper.consume(items);
	}
	virtual void consumeItems(const Tagcoll::OpSet<ITEM>& items, const Tagcoll::OpSet<Tag>& tags)
	{
		tagStripper.consume(items, tags);
	}

public:
	Specials(unsigned int maxPerGroup = 0) :
		tagStripper(coll), maxPerGroup(maxPerGroup), computed(false) {}
	virtual ~Specials() {}
	
	bool isComputed() const { return computed; }
	void compute(Status* tracker = 0);
	void clear()
	{
		coll = Tagcoll::CardinalityStore<ITEM, Facet>(); specials.clear();
		computed = false;
	}

	std::map<Facet, Tagcoll::OpSet<ITEM> > specials;
};

}

// vim:set ts=4 sw=4:
#endif
